<?php

namespace Drupal\symfony_mailer_queue\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides the Symfony mailer queue settings form.
 */
class SymfonyMailerQueueSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'symfony_mailer_queue_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['symfony_mailer_queue.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $config = $this->configFactory->get('symfony_mailer_queue.settings');

    $form['maximum_attempts'] = [
      '#type' => 'number',
      '#min' => 0,
      '#step' => 1,
      '#title' => $this->t('Maximum attempts'),
      '#default_value' => $config->get('maximum_attempts') ?: 5,
      '#description' => $this->t('How many times to retry sending an email before releasing the queue item.'),
      '#required' => TRUE,
    ];

    $form['requeue_delay'] = [
      '#type' => 'number',
      '#min' => 0,
      '#step' => 30,
      '#title' => $this->t('Requeue delay'),
      '#default_value' => $config->get('requeue_delay') ?: 120,
      '#field_suffix' => $this->t('seconds'),
      '#description' => $this->t('How long to wait in seconds before retrying of failed emails.'),
      '#required' => TRUE,
    ];

    $form['send_wait_time'] = [
      '#type' => 'number',
      '#step' => 1,
      '#min' => 0,
      '#title' => $this->t('Wait time per item'),
      '#description' => $this->t('How long to wait in seconds in between the processing of each queue item.'),
      '#field_suffix' => $this->t('seconds'),
      '#default_value' => $config->get('send_wait_time') ?: 1,
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('symfony_mailer_queue.settings')
      ->set('maximum_attempts', $form_state->getValue('maximum_attempts'))
      ->set('requeue_delay', $form_state->getValue('requeue_delay'))
      ->set('send_wait_time', $form_state->getValue('send_wait_time'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
