<?php

namespace Drupal\symfony_mailer_queue\Plugin\EmailAdjuster;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\symfony_mailer\EmailInterface;
use Drupal\symfony_mailer\Exception\SkipMailException;
use Drupal\symfony_mailer\Processor\EmailAdjusterBase;
use Drupal\symfony_mailer_queue\Plugin\QueueWorker\SymfonyMailerQueueWorker;
use Drupal\symfony_mailer_queue\QueueableEmailInterface;
use Drupal\symfony_mailer_queue\SymfonyMailerQueueItem;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines an email adjuster to queue sending of emails.
 *
 * @EmailAdjuster(
 *   id = "queue_sending",
 *   label = @Translation("Queue sending"),
 *   description = @Translation("Queues email instead of immediately sending it."),
 *   weight = 0,
 * )
 */
class QueueSendingEmailAdjuster extends EmailAdjusterBase implements ContainerFactoryPluginInterface {

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected QueueFactory $queueFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, ...$defaults) {
    $instance = new static(...$defaults);
    $instance->queueFactory = $container->get('queue');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build(EmailInterface $email): void {
    if (!$email instanceof QueueableEmailInterface) {
      throw new SkipMailException('Attempted to queue a non-queueable email');
    }
    if (!$email->isInQueue()) {
      $queue = $this->queueFactory->get(SymfonyMailerQueueWorker::QUEUE_NAME, TRUE);
      $item = new SymfonyMailerQueueItem($email->getType(), $email->getSubType(), $email->getParams());
      $queue->createItem($item);
      throw new SkipMailException('The email was queued for sending');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form['description'] = [
      '#markup' => '<p>' . $this->t('Queues email instead of immediately sending it.') . '</p>',
    ];
    return $form;
  }

}
